@@ -1,11 +1,11 @@
 /* C format strings.
-   Copyright (C) 2001-2004 Free Software Foundation, Inc.
+   Copyright (C) 2001-2004, 2006-2007 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,8 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -26,10 +25,8 @@
 #include "format.h"
 #include "c-ctype.h"
 #include "xalloc.h"
-#include "xerror.h"
+#include "xvasprintf.h"
 #include "format-invalid.h"
-#include "error.h"
-#include "error-progname.h"
 #include "gettext.h"
 
 #define _(str) gettext (str)
@@ -136,16 +133,21 @@ enum format_arg_type
 			   | FAT_SIZE_INTMAX_T | FAT_SIZE_INTPTR_T
 			   | FAT_SIZE_SIZE_T | FAT_SIZE_PTRDIFF_T)
 };
+#ifdef __cplusplus
+typedef int format_arg_type_t;
+#else
+typedef enum format_arg_type format_arg_type_t;
+#endif
 
 struct numbered_arg
 {
   unsigned int number;
-  enum format_arg_type type;
+  format_arg_type_t type;
 };
 
 struct unnumbered_arg
 {
-  enum format_arg_type type;
+  format_arg_type_t type;
 };
 
 struct spec
@@ -154,6 +156,7 @@ struct spec
   unsigned int unnumbered_arg_count;
   unsigned int allocated;
   struct unnumbered_arg *unnumbered;
+  bool unlikely_intentional;
   unsigned int sysdep_directives_count;
   const char **sysdep_directives;
 };
@@ -179,8 +182,9 @@ numbered_arg_compare (const void *p1, const void *p2)
 
 static void *
 format_parse (const char *format, bool translated, bool objc_extensions,
-	      char **invalid_reason)
+	      char *fdi, char **invalid_reason)
 {
+  const char *const format_start = format;
   struct spec spec;
   unsigned int numbered_arg_count;
   struct numbered_arg *numbered;
@@ -192,6 +196,7 @@ format_parse (const char *format, bool translated, bool objc_extensions,
   spec.allocated = 0;
   numbered = NULL;
   spec.unnumbered = NULL;
+  spec.unlikely_intentional = false;
   spec.sysdep_directives_count = 0;
   spec.sysdep_directives = NULL;
 
@@ -200,9 +205,10 @@ format_parse (const char *format, bool translated, bool objc_extensions,
       {
 	/* A directive.  */
 	unsigned int number = 0;
-	enum format_arg_type type;
-	enum format_arg_type size;
+	format_arg_type_t type;
+	format_arg_type_t size;
 
+	FDI_SET (format - 1, FMTDIR_START);
 	spec.directives++;
 
 	if (isdigit (*format))
@@ -222,6 +228,7 @@ format_parse (const char *format, bool translated, bool objc_extensions,
 		if (m == 0)
 		  {
 		    *invalid_reason = INVALID_ARGNO_0 (spec.directives);
+		    FDI_SET (f, FMTDIR_ERROR);
 		    goto bad_format;
 		  }
 		number = m;
@@ -276,6 +283,7 @@ format_parse (const char *format, bool translated, bool objc_extensions,
 		      {
 			*invalid_reason =
 			  INVALID_WIDTH_ARGNO_0 (spec.directives);
+			FDI_SET (f, FMTDIR_ERROR);
 			goto bad_format;
 		      }
 		    width_number = m;
@@ -291,6 +299,7 @@ format_parse (const char *format, bool translated, bool objc_extensions,
 		if (spec.unnumbered_arg_count > 0)
 		  {
 		    *invalid_reason = INVALID_MIXES_NUMBERED_UNNUMBERED ();
+		    FDI_SET (format - 1, FMTDIR_ERROR);
 		    goto bad_format;
 		  }
 
@@ -311,6 +320,7 @@ format_parse (const char *format, bool translated, bool objc_extensions,
 		if (numbered_arg_count > 0)
 		  {
 		    *invalid_reason = INVALID_MIXES_NUMBERED_UNNUMBERED ();
+		    FDI_SET (format - 1, FMTDIR_ERROR);
 		    goto bad_format;
 		  }
 
@@ -357,6 +367,7 @@ format_parse (const char *format, bool translated, bool objc_extensions,
 			  {
 			    *invalid_reason =
 			      INVALID_PRECISION_ARGNO_0 (spec.directives);
+			    FDI_SET (f, FMTDIR_ERROR);
 			    goto bad_format;
 			  }
 			precision_number = m;
@@ -372,6 +383,7 @@ format_parse (const char *format, bool translated, bool objc_extensions,
 		    if (spec.unnumbered_arg_count > 0)
 		      {
 			*invalid_reason = INVALID_MIXES_NUMBERED_UNNUMBERED ();
+			FDI_SET (format - 1, FMTDIR_ERROR);
 			goto bad_format;
 		      }
 
@@ -392,6 +404,7 @@ format_parse (const char *format, bool translated, bool objc_extensions,
 		    if (numbered_arg_count > 0)
 		      {
 			*invalid_reason = INVALID_MIXES_NUMBERED_UNNUMBERED ();
+			FDI_SET (format - 1, FMTDIR_ERROR);
 			goto bad_format;
 		      }
 
@@ -427,18 +440,21 @@ format_parse (const char *format, bool translated, bool objc_extensions,
 	    if (*format != 'P')
 	      {
 		*invalid_reason = INVALID_C99_MACRO (spec.directives);
+		FDI_SET (*format == '\0' ? format - 1 : format, FMTDIR_ERROR);
 		goto bad_format;
 	      }
 	    format++;
 	    if (*format != 'R')
 	      {
 		*invalid_reason = INVALID_C99_MACRO (spec.directives);
+		FDI_SET (*format == '\0' ? format - 1 : format, FMTDIR_ERROR);
 		goto bad_format;
 	      }
 	    format++;
 	    if (*format != 'I')
 	      {
 		*invalid_reason = INVALID_C99_MACRO (spec.directives);
+		FDI_SET (*format == '\0' ? format - 1 : format, FMTDIR_ERROR);
 		goto bad_format;
 	      }
 	    format++;
@@ -453,6 +469,7 @@ format_parse (const char *format, bool translated, bool objc_extensions,
 		break;
 	      default:
 		*invalid_reason = INVALID_C99_MACRO (spec.directives);
+		FDI_SET (*format == '\0' ? format - 1 : format, FMTDIR_ERROR);
 		goto bad_format;
 	      }
 	    format++;
@@ -496,6 +513,8 @@ format_parse (const char *format, bool translated, bool objc_extensions,
 		    else
 		      {
 			*invalid_reason = INVALID_C99_MACRO (spec.directives);
+			FDI_SET (*format == '\0' ? format - 1 : format,
+				 FMTDIR_ERROR);
 			goto bad_format;
 		      }
 		  }
@@ -526,6 +545,8 @@ format_parse (const char *format, bool translated, bool objc_extensions,
 		    else
 		      {
 			*invalid_reason = INVALID_C99_MACRO (spec.directives);
+			FDI_SET (*format == '\0' ? format - 1 : format,
+				 FMTDIR_ERROR);
 			goto bad_format;
 		      }
 		  }
@@ -554,6 +575,8 @@ format_parse (const char *format, bool translated, bool objc_extensions,
 		    else
 		      {
 			*invalid_reason = INVALID_C99_MACRO (spec.directives);
+			FDI_SET (*format == '\0' ? format - 1 : format,
+				 FMTDIR_ERROR);
 			goto bad_format;
 		      }
 		  }
@@ -563,6 +586,7 @@ format_parse (const char *format, bool translated, bool objc_extensions,
 	      {
 		*invalid_reason =
 		  xasprintf (_("In the directive number %u, the token after '<' is not followed by '>'."), spec.directives);
+		FDI_SET (*format == '\0' ? format - 1 : format, FMTDIR_ERROR);
 		goto bad_format;
 	      }
 
@@ -610,6 +634,13 @@ format_parse (const char *format, bool translated, bool objc_extensions,
 	    switch (*format)
 	      {
 	      case '%':
+		/* Programmers writing _("%2%") most often will not want to
+		   use this string as a c-format string, but rather as a
+		   literal or as a different kind of format string.  */
+		if (format[-1] != '%')
+		  spec.unlikely_intentional = true;
+		type = FAT_NONE;
+		break;
 	      case 'm': /* glibc extension */
 		type = FAT_NONE;
 		break;
@@ -658,10 +689,17 @@ format_parse (const char *format, bool translated, bool objc_extensions,
 		break;
 	      other:
 	      default:
-		*invalid_reason =
-		  (*format == '\0'
-		   ? INVALID_UNTERMINATED_DIRECTIVE ()
-		   : INVALID_CONVERSION_SPECIFIER (spec.directives, *format));
+		if (*format == '\0')
+		  {
+		    *invalid_reason = INVALID_UNTERMINATED_DIRECTIVE ();
+		    FDI_SET (format - 1, FMTDIR_ERROR);
+		  }
+		else
+		  {
+		    *invalid_reason =
+		      INVALID_CONVERSION_SPECIFIER (spec.directives, *format);
+		    FDI_SET (format, FMTDIR_ERROR);
+		  }
 		goto bad_format;
 	      }
 	  }
@@ -676,6 +714,7 @@ format_parse (const char *format, bool translated, bool objc_extensions,
 		if (spec.unnumbered_arg_count > 0)
 		  {
 		    *invalid_reason = INVALID_MIXES_NUMBERED_UNNUMBERED ();
+		    FDI_SET (format, FMTDIR_ERROR);
 		    goto bad_format;
 		  }
 
@@ -696,6 +735,7 @@ format_parse (const char *format, bool translated, bool objc_extensions,
 		if (numbered_arg_count > 0)
 		  {
 		    *invalid_reason = INVALID_MIXES_NUMBERED_UNNUMBERED ();
+		    FDI_SET (format, FMTDIR_ERROR);
 		    goto bad_format;
 		  }
 
@@ -709,6 +749,8 @@ format_parse (const char *format, bool translated, bool objc_extensions,
 	      }
 	  }
 
+	FDI_SET (format, FMTDIR_END);
+
 	format++;
       }
 
@@ -726,9 +768,9 @@ format_parse (const char *format, bool translated, bool objc_extensions,
       for (i = j = 0; i < numbered_arg_count; i++)
 	if (j > 0 && numbered[i].number == numbered[j-1].number)
 	  {
-	    enum format_arg_type type1 = numbered[i].type;
-	    enum format_arg_type type2 = numbered[j-1].type;
-	    enum format_arg_type type_both;
+	    format_arg_type_t type1 = numbered[i].type;
+	    format_arg_type_t type2 = numbered[j-1].type;
+	    format_arg_type_t type_both;
 
 	    if (type1 == type2)
 	      type_both = type1;
@@ -777,14 +819,14 @@ format_parse (const char *format, bool translated, bool objc_extensions,
 	 of unnumbered arguments.  */
       spec.unnumbered_arg_count = numbered_arg_count;
       spec.allocated = spec.unnumbered_arg_count;
-      spec.unnumbered = (struct unnumbered_arg *) xmalloc (spec.allocated * sizeof (struct unnumbered_arg));
+      spec.unnumbered = XNMALLOC (spec.allocated, struct unnumbered_arg);
       for (i = 0; i < spec.unnumbered_arg_count; i++)
 	spec.unnumbered[i].type = numbered[i].type;
       free (numbered);
       numbered_arg_count = 0;
     }
 
-  result = (struct spec *) xmalloc (sizeof (struct spec));
+  result = XMALLOC (struct spec);
   *result = spec;
   return result;
 
@@ -799,15 +841,17 @@ format_parse (const char *format, bool translated, bool objc_extensions,
 }
 
 static void *
-format_c_parse (const char *format, bool translated, char **invalid_reason)
+format_c_parse (const char *format, bool translated, char *fdi,
+		char **invalid_reason)
 {
-  return format_parse (format, translated, false, invalid_reason);
+  return format_parse (format, translated, false, fdi, invalid_reason);
 }
 
 static void *
-format_objc_parse (const char *format, bool translated, char **invalid_reason)
+format_objc_parse (const char *format, bool translated, char *fdi,
+		   char **invalid_reason)
 {
-  return format_parse (format, translated, true, invalid_reason);
+  return format_parse (format, translated, true, fdi, invalid_reason);
 }
 
 static void
@@ -822,6 +866,14 @@ format_free (void *descr)
   free (spec);
 }
 
+static bool
+format_is_unlikely_intentional (void *descr)
+{
+  struct spec *spec = (struct spec *) descr;
+
+  return spec->unlikely_intentional;
+}
+
 static int
 format_get_number_of_directives (void *descr)
 {
@@ -831,8 +883,9 @@ format_get_number_of_directives (void *descr)
 }
 
 static bool
-format_check (const lex_pos_ty *pos, void *msgid_descr, void *msgstr_descr,
-	      bool equality, bool noisy, const char *pretty_msgstr)
+format_check (void *msgid_descr, void *msgstr_descr, bool equality,
+	      formatstring_error_logger_t error_logger,
+	      const char *pretty_msgstr)
 {
   struct spec *spec1 = (struct spec *) msgid_descr;
   struct spec *spec2 = (struct spec *) msgstr_descr;
@@ -844,28 +897,18 @@ format_check (const lex_pos_ty *pos, void *msgid_descr, void *msgstr_descr,
       ? spec1->unnumbered_arg_count != spec2->unnumbered_arg_count
       : spec1->unnumbered_arg_count < spec2->unnumbered_arg_count)
     {
-      if (noisy)
-	{
-	  error_with_progname = false;
-	  error_at_line (0, 0, pos->file_name, pos->line_number,
-			 _("number of format specifications in 'msgid' and '%s' does not match"),
-			 pretty_msgstr);
-	  error_with_progname = true;
-	}
+      if (error_logger)
+	error_logger (_("number of format specifications in 'msgid' and '%s' does not match"),
+		      pretty_msgstr);
       err = true;
     }
   else
     for (i = 0; i < spec2->unnumbered_arg_count; i++)
       if (spec1->unnumbered[i].type != spec2->unnumbered[i].type)
 	{
-	  if (noisy)
-	    {
-	      error_with_progname = false;
-	      error_at_line (0, 0, pos->file_name, pos->line_number,
-			     _("format specifications in 'msgid' and '%s' for argument %u are not the same"),
-			     pretty_msgstr, i + 1);
-	      error_with_progname = true;
-	    }
+	  if (error_logger)
+	    error_logger (_("format specifications in 'msgid' and '%s' for argument %u are not the same"),
+			  pretty_msgstr, i + 1);
 	  err = true;
 	}
 
@@ -878,6 +921,7 @@ struct formatstring_parser formatstring_c =
   format_c_parse,
   format_free,
   format_get_number_of_directives,
+  format_is_unlikely_intentional,
   format_check
 };
 
@@ -887,6 +931,7 @@ struct formatstring_parser formatstring_objc =
   format_objc_parse,
   format_free,
   format_get_number_of_directives,
+  format_is_unlikely_intentional,
   format_check
 };
 
@@ -900,13 +945,13 @@ get_sysdep_c_format_directives (const char *string, bool translated,
      particular language.)  */
   char *invalid_reason = NULL;
   struct spec *descr =
-    (struct spec *) format_parse (string, translated, true, &invalid_reason);
+    (struct spec *)
+    format_parse (string, translated, true, NULL, &invalid_reason);
 
   if (descr != NULL && descr->sysdep_directives_count > 0)
     {
       unsigned int n = descr->sysdep_directives_count;
-      struct interval *intervals =
-	(struct interval *) xmalloc (n * sizeof (struct interval));
+      struct interval *intervals = XNMALLOC (n, struct interval);
       unsigned int i;
 
       for (i = 0; i < n; i++)
@@ -936,7 +981,6 @@ get_sysdep_c_format_directives (const char *string, bool translated,
    format_parse for strings read from standard input.  */
 
 #include <stdio.h>
-#include "getline.h"
 
 static void
 format_print (void *descr)
@@ -1072,7 +1116,7 @@ main ()
 	line[--line_len] = '\0';
 
       invalid_reason = NULL;
-      descr = format_c_parse (line, false, &invalid_reason);
+      descr = format_c_parse (line, false, NULL, &invalid_reason);
 
       format_print (descr);
       printf ("\n");
@@ -1089,7 +1133,7 @@ main ()
 /*
  * For Emacs M-x compile
  * Local Variables:
- * compile-command: "/bin/sh ../libtool --mode=link gcc -o a.out -static -O -g -Wall -I.. -I../lib -I../intl -DHAVE_CONFIG_H -DTEST format-c.c ../lib/libgettextlib.la"
+ * compile-command: "/bin/sh ../libtool --tag=CC --mode=link gcc -o a.out -static -O -g -Wall -I.. -I../gnulib-lib -I../intl -DHAVE_CONFIG_H -DTEST format-c.c ../gnulib-lib/libgettextlib.la"
  * End:
  */
 
