@@ -1,11 +1,11 @@
 /* librep format strings.
-   Copyright (C) 2001-2003 Free Software Foundation, Inc.
+   Copyright (C) 2001-2004, 2006-2007 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,8 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -26,10 +25,8 @@
 #include "format.h"
 #include "c-ctype.h"
 #include "xalloc.h"
-#include "xerror.h"
+#include "xvasprintf.h"
 #include "format-invalid.h"
-#include "error.h"
-#include "error-progname.h"
 #include "gettext.h"
 
 #define _(str) gettext (str)
@@ -94,8 +91,10 @@ numbered_arg_compare (const void *p1, const void *p2)
 }
 
 static void *
-format_parse (const char *format, char **invalid_reason)
+format_parse (const char *format, bool translated, char *fdi,
+	      char **invalid_reason)
 {
+  const char *const format_start = format;
   struct spec spec;
   struct spec *result;
   unsigned int number;
@@ -112,6 +111,7 @@ format_parse (const char *format, char **invalid_reason)
 	/* A directive.  */
 	enum format_arg_type type;
 
+	FDI_SET (format - 1, FMTDIR_START);
 	spec.directives++;
 
 	if (isdigit (*format))
@@ -173,10 +173,17 @@ format_parse (const char *format, char **invalid_reason)
 	    type = FAT_OBJECT;
 	    break;
 	  default:
-	    *invalid_reason =
-	      (*format == '\0'
-	       ? INVALID_UNTERMINATED_DIRECTIVE ()
-	       : INVALID_CONVERSION_SPECIFIER (spec.directives, *format));
+	    if (*format == '\0')
+	      {
+		*invalid_reason = INVALID_UNTERMINATED_DIRECTIVE ();
+		FDI_SET (format - 1, FMTDIR_ERROR);
+	      }
+	    else
+	      {
+		*invalid_reason =
+		  INVALID_CONVERSION_SPECIFIER (spec.directives, *format);
+		FDI_SET (format, FMTDIR_ERROR);
+	      }
 	    goto bad_format;
 	  }
 
@@ -194,6 +201,8 @@ format_parse (const char *format, char **invalid_reason)
 	    number++;
 	  }
 
+	FDI_SET (format, FMTDIR_END);
+
 	format++;
       }
 
@@ -244,7 +253,7 @@ format_parse (const char *format, char **invalid_reason)
 	goto bad_format;
     }
 
-  result = (struct spec *) xmalloc (sizeof (struct spec));
+  result = XMALLOC (struct spec);
   *result = spec;
   return result;
 
@@ -273,8 +282,9 @@ format_get_number_of_directives (void *descr)
 }
 
 static bool
-format_check (const lex_pos_ty *pos, void *msgid_descr, void *msgstr_descr,
-	      bool equality, bool noisy, const char *pretty_msgstr)
+format_check (void *msgid_descr, void *msgstr_descr, bool equality,
+	      formatstring_error_logger_t error_logger,
+	      const char *pretty_msgstr)
 {
   struct spec *spec1 = (struct spec *) msgid_descr;
   struct spec *spec2 = (struct spec *) msgstr_descr;
@@ -298,14 +308,9 @@ format_check (const lex_pos_ty *pos, void *msgid_descr, void *msgstr_descr,
 
 	  if (cmp > 0)
 	    {
-	      if (noisy)
-		{
-		  error_with_progname = false;
-		  error_at_line (0, 0, pos->file_name, pos->line_number,
-				 _("a format specification for argument %u, as in '%s', doesn't exist in 'msgid'"),
-				 spec2->numbered[j].number, pretty_msgstr);
-		  error_with_progname = true;
-		}
+	      if (error_logger)
+		error_logger (_("a format specification for argument %u, as in '%s', doesn't exist in 'msgid'"),
+			      spec2->numbered[j].number, pretty_msgstr);
 	      err = true;
 	      break;
 	    }
@@ -313,14 +318,9 @@ format_check (const lex_pos_ty *pos, void *msgid_descr, void *msgstr_descr,
 	    {
 	      if (equality)
 		{
-		  if (noisy)
-		    {
-		      error_with_progname = false;
-		      error_at_line (0, 0, pos->file_name, pos->line_number,
-				     _("a format specification for argument %u doesn't exist in '%s'"),
-				     spec1->numbered[i].number, pretty_msgstr);
-		      error_with_progname = true;
-		    }
+		  if (error_logger)
+		    error_logger (_("a format specification for argument %u doesn't exist in '%s'"),
+				  spec1->numbered[i].number, pretty_msgstr);
 		  err = true;
 		  break;
 		}
@@ -338,15 +338,9 @@ format_check (const lex_pos_ty *pos, void *msgid_descr, void *msgstr_descr,
 	      {
 		if (spec1->numbered[i].type != spec2->numbered[j].type)
 		  {
-		    if (noisy)
-		      {
-			error_with_progname = false;
-			error_at_line (0, 0, pos->file_name, pos->line_number,
-				       _("format specifications in 'msgid' and '%s' for argument %u are not the same"),
-				       pretty_msgstr,
-				       spec2->numbered[j].number);
-			error_with_progname = true;
-		      }
+		    if (error_logger)
+		      error_logger (_("format specifications in 'msgid' and '%s' for argument %u are not the same"),
+				    pretty_msgstr, spec2->numbered[j].number);
 		    err = true;
 		    break;
 		  }
@@ -366,6 +360,7 @@ struct formatstring_parser formatstring_librep =
   format_parse,
   format_free,
   format_get_number_of_directives,
+  NULL,
   format_check
 };
 
@@ -376,7 +371,6 @@ struct formatstring_parser formatstring_librep =
    format_parse for strings read from standard input.  */
 
 #include <stdio.h>
-#include "getline.h"
 
 static void
 format_print (void *descr)
@@ -443,7 +437,7 @@ main ()
 	line[--line_len] = '\0';
 
       invalid_reason = NULL;
-      descr = format_parse (line, &invalid_reason);
+      descr = format_parse (line, false, NULL, &invalid_reason);
 
       format_print (descr);
       printf ("\n");
@@ -460,7 +454,7 @@ main ()
 /*
  * For Emacs M-x compile
  * Local Variables:
- * compile-command: "/bin/sh ../libtool --mode=link gcc -o a.out -static -O -g -Wall -I.. -I../lib -I../intl -DHAVE_CONFIG_H -DTEST format-librep.c ../lib/libgettextlib.la"
+ * compile-command: "/bin/sh ../libtool --tag=CC --mode=link gcc -o a.out -static -O -g -Wall -I.. -I../gnulib-lib -I../intl -DHAVE_CONFIG_H -DTEST format-librep.c ../gnulib-lib/libgettextlib.la"
  * End:
  */
 
