@@ -1,11 +1,11 @@
 /* Python format strings.
-   Copyright (C) 2001-2004 Free Software Foundation, Inc.
+   Copyright (C) 2001-2004, 2006-2007 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,8 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -27,7 +26,7 @@
 #include "format.h"
 #include "c-ctype.h"
 #include "xalloc.h"
-#include "xerror.h"
+#include "xvasprintf.h"
 #include "format-invalid.h"
 #include "gettext.h"
 
@@ -113,8 +112,10 @@ named_arg_compare (const void *p1, const void *p2)
   xstrdup (_("The string refers to arguments both through argument names and through unnamed argument specifications."))
 
 static void *
-format_parse (const char *format, bool translated, char **invalid_reason)
+format_parse (const char *format, bool translated, char *fdi,
+	      char **invalid_reason)
 {
+  const char *const format_start = format;
   struct spec spec;
   struct spec *result;
 
@@ -132,6 +133,7 @@ format_parse (const char *format, bool translated, char **invalid_reason)
 	char *name = NULL;
 	enum format_arg_type type;
 
+	FDI_SET (format - 1, FMTDIR_START);
 	spec.directives++;
 
 	if (*format == '(')
@@ -158,12 +160,13 @@ format_parse (const char *format, bool translated, char **invalid_reason)
 	    if (*format == '\0')
 	      {
 		*invalid_reason = INVALID_UNTERMINATED_DIRECTIVE ();
+		FDI_SET (format - 1, FMTDIR_ERROR);
 		goto bad_format;
 	      }
 	    name_end = format++;
 
 	    n = name_end - name_start;
-	    name = (char *) xmalloc (n + 1);
+	    name = XNMALLOC (n + 1, char);
 	    memcpy (name, name_start, n);
 	    name[n] = '\0';
 	  }
@@ -180,6 +183,7 @@ format_parse (const char *format, bool translated, char **invalid_reason)
 	    if (spec.named_arg_count > 0)
 	      {
 		*invalid_reason = INVALID_MIXES_NAMED_UNNAMED ();
+		FDI_SET (format - 1, FMTDIR_ERROR);
 		goto bad_format;
 	      }
 
@@ -208,6 +212,7 @@ format_parse (const char *format, bool translated, char **invalid_reason)
 		if (spec.named_arg_count > 0)
 		  {
 		    *invalid_reason = INVALID_MIXES_NAMED_UNNAMED ();
+		    FDI_SET (format - 1, FMTDIR_ERROR);
 		    goto bad_format;
 		  }
 
@@ -246,10 +251,17 @@ format_parse (const char *format, bool translated, char **invalid_reason)
 	    type = FAT_FLOAT;
 	    break;
 	  default:
-	    *invalid_reason =
-	      (*format == '\0'
-	       ? INVALID_UNTERMINATED_DIRECTIVE ()
-	       : INVALID_CONVERSION_SPECIFIER (spec.directives, *format));
+	    if (*format == '\0')
+	      {
+		*invalid_reason = INVALID_UNTERMINATED_DIRECTIVE ();
+		FDI_SET (format - 1, FMTDIR_ERROR);
+	      }
+	    else
+	      {
+		*invalid_reason =
+		  INVALID_CONVERSION_SPECIFIER (spec.directives, *format);
+		FDI_SET (format, FMTDIR_ERROR);
+	      }
 	    goto bad_format;
 	  }
 
@@ -261,6 +273,7 @@ format_parse (const char *format, bool translated, char **invalid_reason)
 	    if (spec.unnamed_arg_count > 0)
 	      {
 		*invalid_reason = INVALID_MIXES_NAMED_UNNAMED ();
+		FDI_SET (format, FMTDIR_ERROR);
 		goto bad_format;
 	      }
 
@@ -281,6 +294,7 @@ format_parse (const char *format, bool translated, char **invalid_reason)
 	    if (spec.named_arg_count > 0)
 	      {
 		*invalid_reason = INVALID_MIXES_NAMED_UNNAMED ();
+		FDI_SET (format, FMTDIR_ERROR);
 		goto bad_format;
 	      }
 
@@ -293,6 +307,8 @@ format_parse (const char *format, bool translated, char **invalid_reason)
 	    spec.unnamed_arg_count++;
 	  }
 
+	FDI_SET (format, FMTDIR_END);
+
 	format++;
       }
 
@@ -346,7 +362,7 @@ format_parse (const char *format, bool translated, char **invalid_reason)
 	goto bad_format;
     }
 
-  result = (struct spec *) xmalloc (sizeof (struct spec));
+  result = XMALLOC (struct spec);
   *result = spec;
   return result;
 
@@ -507,17 +523,40 @@ struct formatstring_parser formatstring_python =
   format_parse,
   format_free,
   format_get_number_of_directives,
+  NULL,
   format_check
 };
 
 
+unsigned int
+get_python_format_unnamed_arg_count (const char *string)
+{
+  /* Parse the format string.  */
+  char *invalid_reason = NULL;
+  struct spec *descr =
+    (struct spec *) format_parse (string, false, NULL, &invalid_reason);
+
+  if (descr != NULL)
+    {
+      unsigned int result = descr->unnamed_arg_count;
+
+      format_free (descr);
+      return result;
+    }
+  else
+    {
+      free (invalid_reason);
+      return 0;
+    }
+}
+
+
 #ifdef TEST
 
 /* Test program: Print the argument list specification returned by
    format_parse for strings read from standard input.  */
 
 #include <stdio.h>
-#include "getline.h"
 
 static void
 format_print (void *descr)
@@ -615,7 +654,7 @@ main ()
 	line[--line_len] = '\0';
 
       invalid_reason = NULL;
-      descr = format_parse (line, false, &invalid_reason);
+      descr = format_parse (line, false, NULL, &invalid_reason);
 
       format_print (descr);
       printf ("\n");
@@ -632,7 +671,7 @@ main ()
 /*
  * For Emacs M-x compile
  * Local Variables:
- * compile-command: "/bin/sh ../libtool --mode=link gcc -o a.out -static -O -g -Wall -I.. -I../lib -I../intl -DHAVE_CONFIG_H -DTEST format-python.c ../lib/libgettextlib.la"
+ * compile-command: "/bin/sh ../libtool --tag=CC --mode=link gcc -o a.out -static -O -g -Wall -I.. -I../gnulib-lib -I../intl -DHAVE_CONFIG_H -DTEST format-python.c ../gnulib-lib/libgettextlib.la"
  * End:
  */
 
