@@ -1,11 +1,11 @@
 /* GNU gettext - internationalization aids
-   Copyright (C) 1995-1998, 2000-2005 Free Software Foundation, Inc.
+   Copyright (C) 1995-1998, 2000-2007 Free Software Foundation, Inc.
    This file was written by Peter Miller <millerp@canb.auug.org.au>
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,8 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -35,11 +34,14 @@
 #include "relocatable.h"
 #include "basename.h"
 #include "message.h"
-#include "exit.h"
+#include "read-catalog.h"
 #include "read-po.h"
+#include "read-properties.h"
+#include "read-stringtable.h"
 #include "msgl-iconv.h"
-#include "strstr.h"
+#include "c-strstr.h"
 #include "c-strcase.h"
+#include "propername.h"
 #include "gettext.h"
 
 #define _(str) gettext (str)
@@ -48,6 +50,12 @@
 /* Apply the .pot file to each of the domains in the PO file.  */
 static bool multi_domain_mode = false;
 
+/* Whether to consider fuzzy messages as translations.  */
+static bool include_fuzzies = false;
+
+/* Whether to consider untranslated messages as translations.  */
+static bool include_untranslated = false;
+
 /* Long options.  */
 static const struct option long_options[] =
 {
@@ -56,6 +64,8 @@ static const struct option long_options[] =
   { "multi-domain", no_argument, NULL, 'm' },
   { "properties-input", no_argument, NULL, 'P' },
   { "stringtable-input", no_argument, NULL, CHAR_MAX + 1 },
+  { "use-fuzzy", no_argument, NULL, CHAR_MAX + 2 },
+  { "use-untranslated", no_argument, NULL, CHAR_MAX + 3 },
   { "version", no_argument, NULL, 'V' },
   { NULL, 0, NULL, 0 }
 };
@@ -67,7 +77,8 @@ static void usage (int status)
 	__attribute__ ((noreturn))
 #endif
 ;
-static void compare (const char *fn1, const char *fn2);
+static void compare (const char *fn1, const char *fn2,
+		     catalog_input_format_ty input_syntax);
 
 
 int
@@ -76,6 +87,7 @@ main (int argc, char *argv[])
   int optchar;
   bool do_help;
   bool do_version;
+  catalog_input_format_ty input_syntax = &input_format_po;
 
   /* Set program name for messages.  */
   set_program_name (argv[0]);
@@ -89,6 +101,7 @@ main (int argc, char *argv[])
 
   /* Set the text message domain.  */
   bindtextdomain (PACKAGE, relocate (LOCALEDIR));
+  bindtextdomain ("bison-runtime", relocate (BISON_LOCALEDIR));
   textdomain (PACKAGE);
 
   /* Ensure that write errors on stdout are detected.  */
@@ -116,15 +129,23 @@ main (int argc, char *argv[])
 	break;
 
       case 'P':
-	input_syntax = syntax_properties;
+	input_syntax = &input_format_properties;
 	break;
 
       case 'V':
 	do_version = true;
 	break;
 
-      case CHAR_MAX + 1: /* --stringtable-input */
-	input_syntax = syntax_stringtable;
+      case CHAR_MAX + 1:	/* --stringtable-input */
+	input_syntax = &input_format_stringtable;
+	break;
+
+      case CHAR_MAX + 2:	/* --use-fuzzy */
+	include_fuzzies = true;
+	break;
+
+      case CHAR_MAX + 3:	/* --use-untranslated */
+	include_untranslated = true;
 	break;
 
       default:
@@ -138,11 +159,12 @@ main (int argc, char *argv[])
       printf ("%s (GNU %s) %s\n", basename (program_name), PACKAGE, VERSION);
       /* xgettext: no-wrap */
       printf (_("Copyright (C) %s Free Software Foundation, Inc.\n\
-This is free software; see the source for copying conditions.  There is NO\n\
-warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
+License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n\
+This is free software: you are free to change and redistribute it.\n\
+There is NO WARRANTY, to the extent permitted by law.\n\
 "),
-	      "1995-1998, 2000-2005");
-      printf (_("Written by %s.\n"), "Peter Miller");
+	      "1995-1998, 2000-2007");
+      printf (_("Written by %s.\n"), proper_name ("Peter Miller"));
       exit (EXIT_SUCCESS);
     }
 
@@ -163,7 +185,7 @@ warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
     }
 
   /* compare the two files */
-  compare (argv[optind], argv[optind + 1]);
+  compare (argv[optind], argv[optind + 1], input_syntax);
   exit (EXIT_SUCCESS);
 }
 
@@ -207,6 +229,10 @@ Input file location:\n"));
 Operation modifiers:\n"));
       printf (_("\
   -m, --multi-domain          apply ref.pot to each of the domains in def.po\n"));
+      printf (_("\
+      --use-fuzzy             consider fuzzy entries\n"));
+      printf (_("\
+      --use-untranslated      consider untranslated entries\n"));
       printf ("\n");
       printf (_("\
 Input file syntax:\n"));
@@ -223,6 +249,10 @@ Informative output:\n"));
       printf (_("\
   -V, --version               output version information and exit\n"));
       printf ("\n");
+      /* TRANSLATORS: The placeholder indicates the bug-reporting address
+         for this package.  Please add _another line_ saying
+         "Report translation bugs to <...>\n" with the address for translation
+         bugs (typically your translation team's web or email address).  */
       fputs (_("Report bugs to <bug-gnu-gettext@gnu.org>.\n"), stdout);
     }
 
@@ -235,7 +265,7 @@ static bool
 is_message_selected (const message_ty *mp)
 {
   /* Always keep the header entry.  */
-  if (mp->msgid[0] == '\0')
+  if (is_header (mp))
     return true;
 
   return !mp->obsolete;
@@ -270,20 +300,37 @@ match_domain (const char *fn1, const char *fn2,
       refmsg = refmlp->item[j];
 
       /* See if it is in the other file.  */
-      defmsg = message_list_search (defmlp, refmsg->msgid);
+      defmsg = message_list_search (defmlp, refmsg->msgctxt, refmsg->msgid);
       if (defmsg)
-	defmsg->used = 1;
+	{
+	  if (!include_untranslated && defmsg->msgstr[0] == '\0')
+	    {
+	      (*nerrors)++;
+	      po_gram_error_at_line (&defmsg->pos, _("\
+this message is untranslated"));
+	    }
+	  else if (!include_fuzzies && defmsg->is_fuzzy && !is_header (defmsg))
+	    {
+	      (*nerrors)++;
+	      po_gram_error_at_line (&defmsg->pos, _("\
+this message needs to be reviewed by the translator"));
+	    }
+	  else
+	    defmsg->used = 1;
+	}
       else
 	{
 	  /* If the message was not defined at all, try to find a very
 	     similar message, it could be a typo, or the suggestion may
 	     help.  */
 	  (*nerrors)++;
-	  defmsg = message_list_search_fuzzy (defmlp, refmsg->msgid);
+	  defmsg =
+	    message_list_search_fuzzy (defmlp, refmsg->msgctxt, refmsg->msgid);
 	  if (defmsg)
 	    {
 	      po_gram_error_at_line (&refmsg->pos, _("\
 this message is used but not defined..."));
+	      error_message_count--;
 	      po_gram_error_at_line (&defmsg->pos, _("\
 ...but this definition is similar"));
 	      defmsg->used = 1;
@@ -297,7 +344,7 @@ this message is used but not defined in %s"), fn1);
 
 
 static void
-compare (const char *fn1, const char *fn2)
+compare (const char *fn1, const char *fn2, catalog_input_format_ty input_syntax)
 {
   msgdomain_list_ty *def;
   msgdomain_list_ty *ref;
@@ -306,11 +353,11 @@ compare (const char *fn1, const char *fn2)
   message_list_ty *empty_list;
 
   /* This is the master file, created by a human.  */
-  def = remove_obsoletes (read_po_file (fn1));
+  def = remove_obsoletes (read_catalog_file (fn1, input_syntax));
 
   /* This is the generated file, created by groping the sources with
      the xgettext program.  */
-  ref = remove_obsoletes (read_po_file (fn2));
+  ref = remove_obsoletes (read_catalog_file (fn2, input_syntax));
 
   /* The references file can be either in ASCII or in UTF-8.  If it is
      in UTF-8, we have to convert the definitions to UTF-8 as well.  */
@@ -321,13 +368,13 @@ compare (const char *fn1, const char *fn2)
 	message_list_ty *mlp = ref->item[k]->messages;
 
 	for (j = 0; j < mlp->nitems; j++)
-	  if (mlp->item[j]->msgid[0] == '\0' /* && !mlp->item[j]->obsolete */)
+	  if (is_header (mlp->item[j]) /* && !mlp->item[j]->obsolete */)
 	    {
 	      const char *header = mlp->item[j]->msgstr;
 
 	      if (header != NULL)
 		{
-		  const char *charsetstr = strstr (header, "charset=");
+		  const char *charsetstr = c_strstr (header, "charset=");
 
 		  if (charsetstr != NULL)
 		    {
@@ -343,7 +390,7 @@ compare (const char *fn1, const char *fn2)
 	    }
 	}
     if (was_utf8)
-      def = iconv_msgdomain_list (def, "UTF-8", fn1);
+      def = iconv_msgdomain_list (def, "UTF-8", true, fn1);
   }
 
   empty_list = message_list_alloc (false);
