@@ -1,11 +1,11 @@
 /* Reading NeXTstep/GNUstep .strings files.
-   Copyright (C) 2003, 2005 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2005-2007 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2003.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,8 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -32,11 +31,11 @@
 
 #include "error.h"
 #include "error-progname.h"
-#include "read-po-abstract.h"
+#include "read-catalog-abstract.h"
 #include "xalloc.h"
-#include "exit.h"
-#include "utf8-ucs4.h"
-#include "ucs4-utf8.h"
+#include "xvasprintf.h"
+#include "po-xerror.h"
+#include "unistr.h"
 #include "gettext.h"
 
 #define _(str) gettext (str)
@@ -92,8 +91,14 @@ phase1_getc ()
   if (c == EOF)
     {
       if (ferror (fp))
-	error (EXIT_FAILURE, errno, _("error while reading \"%s\""),
-	       real_file_name);
+	{
+	  const char *errno_description = strerror (errno);
+	  po_xerror (PO_SEVERITY_FATAL_ERROR, NULL, NULL, 0, 0, false,
+		     xasprintf ("%s: %s",
+				xasprintf (_("error while reading \"%s\""),
+					   real_file_name),
+				errno_description));
+	}
       return EOF;
     }
 
@@ -229,52 +234,46 @@ phase2_getc ()
 	      return UEOF;
 	    buf[1] = c;
 	    count = 2;
-	  }
 
-	if (buf[0] >= 0xe0
-	    && ((buf[1] ^ 0x80) < 0x40))
-	  {
-	    c = phase1_getc ();
-	    if (c == EOF)
-	      return UEOF;
-	    buf[2] = c;
-	    count = 3;
-	  }
+	    if (buf[0] >= 0xe0
+		&& ((buf[1] ^ 0x80) < 0x40))
+	      {
+		c = phase1_getc ();
+		if (c == EOF)
+		  return UEOF;
+		buf[2] = c;
+		count = 3;
 
-	if (buf[0] >= 0xf0
-	    && ((buf[1] ^ 0x80) < 0x40)
-	    && ((buf[2] ^ 0x80) < 0x40))
-	  {
-	    c = phase1_getc ();
-	    if (c == EOF)
-	      return UEOF;
-	    buf[3] = c;
-	    count = 4;
-	  }
+		if (buf[0] >= 0xf0
+		    && ((buf[2] ^ 0x80) < 0x40))
+		  {
+		    c = phase1_getc ();
+		    if (c == EOF)
+		      return UEOF;
+		    buf[3] = c;
+		    count = 4;
 
-	if (buf[0] >= 0xf8
-	    && ((buf[1] ^ 0x80) < 0x40)
-	    && ((buf[2] ^ 0x80) < 0x40)
-	    && ((buf[3] ^ 0x80) < 0x40))
-	  {
-	    c = phase1_getc ();
-	    if (c == EOF)
-	      return UEOF;
-	    buf[4] = c;
-	    count = 5;
-	  }
+		    if (buf[0] >= 0xf8
+			&& ((buf[3] ^ 0x80) < 0x40))
+		      {
+			c = phase1_getc ();
+			if (c == EOF)
+			  return UEOF;
+			buf[4] = c;
+			count = 5;
 
-	if (buf[0] >= 0xfc
-	    && ((buf[1] ^ 0x80) < 0x40)
-	    && ((buf[2] ^ 0x80) < 0x40)
-	    && ((buf[3] ^ 0x80) < 0x40)
-	    && ((buf[4] ^ 0x80) < 0x40))
-	  {
-	    c = phase1_getc ();
-	    if (c == EOF)
-	      return UEOF;
-	    buf[5] = c;
-	    count = 6;
+			if (buf[0] >= 0xfc
+			    && ((buf[4] ^ 0x80) < 0x40))
+			  {
+			    c = phase1_getc ();
+			    if (c == EOF)
+			      return UEOF;
+			    buf[5] = c;
+			    count = 6;
+			  }
+		      }
+		  }
+	      }
 	  }
 
 	u8_mbtouc (&uc, buf, count);
@@ -335,7 +334,7 @@ conv_from_ucs4 (const int *buffer, size_t buflen)
   unsigned char *q;
 
   /* Each UCS-4 word needs 6 bytes at worst.  */
-  utf8_string = (unsigned char *) xmalloc (6 * buflen + 1);
+  utf8_string = XNMALLOC (6 * buflen + 1, unsigned char);
 
   for (pos = 0, q = utf8_string; pos < buflen; )
     {
@@ -798,23 +797,17 @@ read_string (lex_pos_ty *pos)
 	  buffer[buflen++] = c;
 	}
       if (c == UEOF)
-	{
-	  error_with_progname = false;
-	  error (0, 0, _("%s:%lu: warning: unterminated string"),
-		 real_file_name, (unsigned long) gram_pos.line_number);
-	  error_with_progname = true;
-	}
+	po_xerror (PO_SEVERITY_ERROR, NULL,
+		   real_file_name, gram_pos.line_number, (size_t)(-1), false,
+		   _("warning: unterminated string"));
     }
   else
     {
       /* Read a token outside quotes.  */
       if (is_quotable (c))
-	{
-	  error_with_progname = false;
-	  error (0, 0, _("%s:%lu: warning: syntax error"),
-		 real_file_name, (unsigned long) gram_pos.line_number);
-	  error_with_progname = true;
-	}
+	po_xerror (PO_SEVERITY_ERROR, NULL,
+		   real_file_name, gram_pos.line_number, (size_t)(-1), false,
+		   _("warning: syntax error"));
       for (; c != UEOF && !is_quotable (c); c = phase4_getc ())
 	{
 	  if (buflen >= bufmax)
@@ -831,9 +824,9 @@ read_string (lex_pos_ty *pos)
 
 
 /* Read a .strings file from a stream, and dispatch to the various
-   abstract_po_reader_class_ty methods.  */
-void
-stringtable_parse (abstract_po_reader_ty *pop, FILE *file,
+   abstract_catalog_reader_class_ty methods.  */
+static void
+stringtable_parse (abstract_catalog_reader_ty *pop, FILE *file,
 		   const char *real_filename, const char *logical_filename)
 {
   fp = file;
@@ -873,20 +866,20 @@ stringtable_parse (abstract_po_reader_ty *pop, FILE *file,
       /* Expect a '=' or ';'.  */
       if (c == UEOF)
 	{
-	  error_with_progname = false;
-	  error (0, 0, _("%s:%lu: warning: unterminated key/value pair"),
-		 real_file_name, (unsigned long) gram_pos.line_number);
-	  error_with_progname = true;
+	  po_xerror (PO_SEVERITY_ERROR, NULL,
+		     real_file_name, gram_pos.line_number, (size_t)(-1), false,
+		     _("warning: unterminated key/value pair"));
 	  break;
 	}
       if (c == ';')
 	{
 	  /* "key"; is an abbreviation for "key"=""; and does not
 	     necessarily designate an untranslated entry.  */
-	  msgstr = "";
+	  msgstr = xstrdup ("");
 	  msgstr_pos = msgid_pos;
-	  po_callback_message (msgid, &msgid_pos, NULL,
+	  po_callback_message (NULL, msgid, &msgid_pos, NULL,
 			       msgstr, strlen (msgstr) + 1, &msgstr_pos,
+			       NULL, NULL, NULL,
 			       false, next_is_obsolete);
 	}
       else if (c == '=')
@@ -895,10 +888,9 @@ stringtable_parse (abstract_po_reader_ty *pop, FILE *file,
 	  msgstr = read_string (&msgstr_pos);
 	  if (msgstr == NULL)
 	    {
-	      error_with_progname = false;
-	      error (0, 0, _("%s:%lu: warning: unterminated key/value pair"),
-		     real_file_name, (unsigned long) gram_pos.line_number);
-	      error_with_progname = true;
+	      po_xerror (PO_SEVERITY_ERROR, NULL,
+			 real_file_name, gram_pos.line_number, (size_t)(-1),
+			 false, _("warning: unterminated key/value pair"));
 	      break;
 	    }
 
@@ -935,27 +927,26 @@ stringtable_parse (abstract_po_reader_ty *pop, FILE *file,
 		msgstr = fuzzy_msgstr;
 
 	      /* A key/value pair.  */
-	      po_callback_message (msgid, &msgid_pos, NULL,
+	      po_callback_message (NULL, msgid, &msgid_pos, NULL,
 				   msgstr, strlen (msgstr) + 1, &msgstr_pos,
+				   NULL, NULL, NULL,
 				   false, next_is_obsolete);
 	    }
 	  else
 	    {
-	      error_with_progname = false;
-	      error (0, 0, _("\
-%s:%lu: warning: syntax error, expected ';' after string"),
-		     real_file_name, (unsigned long) gram_pos.line_number);
-	      error_with_progname = true;
+	      po_xerror (PO_SEVERITY_ERROR, NULL,
+			 real_file_name, gram_pos.line_number, (size_t)(-1),
+			 false, _("\
+warning: syntax error, expected ';' after string"));
 	      break;
 	    }
 	}
       else
 	{
-	  error_with_progname = false;
-	  error (0, 0, _("\
-%s:%lu: warning: syntax error, expected '=' or ';' after string"),
-		 real_file_name, (unsigned long) gram_pos.line_number);
-	  error_with_progname = true;
+	  po_xerror (PO_SEVERITY_ERROR, NULL,
+		     real_file_name, gram_pos.line_number, (size_t)(-1), false,
+		     _("\
+warning: syntax error, expected '=' or ';' after string"));
 	  break;
 	}
     }
@@ -964,3 +955,9 @@ stringtable_parse (abstract_po_reader_ty *pop, FILE *file,
   real_file_name = NULL;
   gram_pos.line_number = 0;
 }
+
+const struct catalog_input_format input_format_stringtable =
+{
+  stringtable_parse,			/* parse */
+  true					/* produces_utf8 */
+};
