@@ -1,11 +1,11 @@
 /* Reading tcl/msgcat .msg files.
-   Copyright (C) 2002-2003, 2005 Free Software Foundation, Inc.
+   Copyright (C) 2002-2003, 2005-2007 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2002.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,8 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -30,14 +29,14 @@
 
 #include "msgunfmt.h"
 #include "relocatable.h"
-#include "pathname.h"
+#include "filename.h"
 #include "sh-quote.h"
 #include "pipe.h"
 #include "wait-process.h"
+#include "read-catalog.h"
 #include "read-po.h"
-#include "xallocsa.h"
+#include "xmalloca.h"
 #include "error.h"
-#include "exit.h"
 #include "gettext.h"
 
 #define _(str) gettext (str)
@@ -70,11 +69,11 @@ msgdomain_read_tcl (const char *locale_name, const char *directory)
   if (gettextdatadir == NULL || gettextdatadir[0] == '\0')
     gettextdatadir = relocate (GETTEXTDATADIR);
 
-  tclscript = concatenated_pathname (gettextdatadir, "msgunfmt.tcl", NULL);
+  tclscript = concatenated_filename (gettextdatadir, "msgunfmt.tcl", NULL);
 
   /* Convert the locale name to lowercase and remove any encoding.  */
   len = strlen (locale_name);
-  frobbed_locale_name = (char *) xallocsa (len + 1);
+  frobbed_locale_name = (char *) xmalloca (len + 1);
   memcpy (frobbed_locale_name, locale_name, len + 1);
   for (p = frobbed_locale_name; *p != '\0'; p++)
     if (*p >= 'A' && *p <= 'Z')
@@ -85,9 +84,9 @@ msgdomain_read_tcl (const char *locale_name, const char *directory)
 	break;
       }
 
-  file_name = concatenated_pathname (directory, frobbed_locale_name, ".msg");
+  file_name = concatenated_filename (directory, frobbed_locale_name, ".msg");
 
-  freesa (frobbed_locale_name);
+  freea (frobbed_locale_name);
 
   /* Prepare arguments.  */
   argv[0] = "tclsh";
@@ -111,7 +110,7 @@ msgdomain_read_tcl (const char *locale_name, const char *directory)
     error (EXIT_FAILURE, errno, _("fdopen() failed"));
 
   /* Read the message list.  */
-  mdlp = read_po (fp, "(pipe)", "(pipe)");
+  mdlp = read_catalog_stream (fp, "(pipe)", "(pipe)", &input_format_po);
 
   fclose (fp);
 
