@@ -1,12 +1,12 @@
 /* xgettext PO and JavaProperties backends.
-   Copyright (C) 1995-1998, 2000-2003, 2005 Free Software Foundation, Inc.
+   Copyright (C) 1995-1998, 2000-2003, 2005-2006 Free Software Foundation, Inc.
 
    This file was written by Peter Miller <millerp@canb.auug.org.au>
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,13 +14,17 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
 
+/* Specification.  */
+#include "x-po.h"
+#include "x-properties.h"
+#include "x-stringtable.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdbool.h>
@@ -32,7 +36,10 @@
 #include "x-properties.h"
 #include "x-stringtable.h"
 #include "xalloc.h"
+#include "read-catalog.h"
 #include "read-po.h"
+#include "read-properties.h"
+#include "read-stringtable.h"
 #include "po-lex.h"
 #include "gettext.h"
 
@@ -43,16 +50,19 @@
 /* The charset found in the header entry.  */
 static char *header_charset;
 
-/* Define a subclass extract_po_reader_ty of default_po_reader_ty.  */
+/* Define a subclass extract_catalog_reader_ty of default_catalog_reader_ty.  */
 
 static void
-extract_add_message (default_po_reader_ty *this,
+extract_add_message (default_catalog_reader_ty *this,
 		     char *msgctxt,
 		     char *msgid,
 		     lex_pos_ty *msgid_pos,
 		     char *msgid_plural,
 		     char *msgstr, size_t msgstr_len,
 		     lex_pos_ty *msgstr_pos,
+		     char *prev_msgctxt,
+		     char *prev_msgid,
+		     char *prev_msgid_plural,
 		     bool force_fuzzy, bool obsolete)
 {
   /* See whether we shall exclude this message.  */
@@ -65,35 +75,47 @@ extract_add_message (default_po_reader_ty *this,
      because the old header may contain a charset= directive.  */
   if (msgctxt == NULL && *msgid == '\0' && !xgettext_omit_header)
     {
-      const char *charsetstr = strstr (msgstr, "charset=");
+      {
+	const char *charsetstr = strstr (msgstr, "charset=");
 
-      if (charsetstr != NULL)
-	{
-	  size_t len;
-	  char *charset;
+	if (charsetstr != NULL)
+	  {
+	    size_t len;
+	    char *charset;
 
-	  charsetstr += strlen ("charset=");
-	  len = strcspn (charsetstr, " \t\n");
-	  charset = (char *) xmalloc (len + 1);
-	  memcpy (charset, charsetstr, len);
-	  charset[len] = '\0';
+	    charsetstr += strlen ("charset=");
+	    len = strcspn (charsetstr, " \t\n");
+	    charset = XNMALLOC (len + 1, char);
+	    memcpy (charset, charsetstr, len);
+	    charset[len] = '\0';
 
-	  if (header_charset != NULL)
-	    free (header_charset);
-	  header_charset = charset;
-	}
+	    if (header_charset != NULL)
+	      free (header_charset);
+	    header_charset = charset;
+	  }
+      }
 
      discard:
       if (msgctxt != NULL)
 	free (msgctxt);
       free (msgid);
+      if (msgid_plural != NULL)
+	free (msgid_plural);
       free (msgstr);
+      if (prev_msgctxt != NULL)
+	free (prev_msgctxt);
+      if (prev_msgid != NULL)
+	free (prev_msgid);
+      if (prev_msgid_plural != NULL)
+	free (prev_msgid_plural);
       return;
     }
 
   /* Invoke superclass method.  */
   default_add_message (this, msgctxt, msgid, msgid_pos, msgid_plural,
-		       msgstr, msgstr_len, msgstr_pos, force_fuzzy, obsolete);
+		       msgstr, msgstr_len, msgstr_pos,
+		       prev_msgctxt, prev_msgid, prev_msgid_plural,
+		       force_fuzzy, obsolete);
 }
 
 
@@ -103,10 +125,10 @@ extract_add_message (default_po_reader_ty *this,
    and all actions resulting from the parse will be through
    invocations of method functions of that object.  */
 
-static default_po_reader_class_ty extract_methods =
+static default_catalog_reader_class_ty extract_methods =
 {
   {
-    sizeof (default_po_reader_ty),
+    sizeof (default_catalog_reader_ty),
     default_constructor,
     default_destructor,
     default_parse_brief,
@@ -127,14 +149,14 @@ static default_po_reader_class_ty extract_methods =
 static void
 extract (FILE *fp,
 	 const char *real_filename, const char *logical_filename,
-	 input_syntax_ty syntax,
+	 catalog_input_format_ty input_syntax,
 	 msgdomain_list_ty *mdlp)
 {
-  default_po_reader_ty *pop;
+  default_catalog_reader_ty *pop;
 
   header_charset = NULL;
 
-  pop = default_po_reader_alloc (&extract_methods);
+  pop = default_catalog_reader_alloc (&extract_methods);
   pop->handle_comments = true;
   pop->handle_filepos_comments = (line_comment != 0);
   pop->allow_domain_directives = false;
@@ -142,9 +164,9 @@ extract (FILE *fp,
   pop->allow_duplicates_if_same_msgstr = true;
   pop->mdlp = NULL;
   pop->mlp = mdlp->item[0]->messages;
-  po_scan ((abstract_po_reader_ty *) pop, fp, real_filename, logical_filename,
-	   syntax);
-  po_reader_free ((abstract_po_reader_ty *) pop);
+  catalog_reader_parse ((abstract_catalog_reader_ty *) pop, fp, real_filename,
+			logical_filename, input_syntax);
+  catalog_reader_free ((abstract_catalog_reader_ty *) pop);
 
   if (header_charset != NULL)
     {
@@ -173,7 +195,7 @@ extract (FILE *fp,
 		      len1 = charsetstr - header;
 		      len2 = strlen (header_charset);
 		      len3 = (header + strlen (header)) - (charsetstr + len);
-		      new_header = (char *) xmalloc (len1 + len2 + len3 + 1);
+		      new_header = XNMALLOC (len1 + len2 + len3 + 1, char);
 		      memcpy (new_header, header, len1);
 		      memcpy (new_header + len1, header_charset, len2);
 		      memcpy (new_header + len1 + len2, charsetstr + len, len3 + 1);
@@ -195,7 +217,7 @@ extract_po (FILE *fp,
 	    flag_context_list_table_ty *flag_table,
 	    msgdomain_list_ty *mdlp)
 {
-  extract (fp, real_filename,  logical_filename, syntax_po, mdlp);
+  extract (fp, real_filename,  logical_filename, &input_format_po, mdlp);
 }
 
 
@@ -205,7 +227,8 @@ extract_properties (FILE *fp,
 		    flag_context_list_table_ty *flag_table,
 		    msgdomain_list_ty *mdlp)
 {
-  extract (fp, real_filename,  logical_filename, syntax_properties, mdlp);
+  extract (fp, real_filename,  logical_filename, &input_format_properties,
+	   mdlp);
 }
 
 
@@ -215,5 +238,6 @@ extract_stringtable (FILE *fp,
 		     flag_context_list_table_ty *flag_table,
 		     msgdomain_list_ty *mdlp)
 {
-  extract (fp, real_filename,  logical_filename, syntax_stringtable, mdlp);
+  extract (fp, real_filename,  logical_filename, &input_format_stringtable,
+	   mdlp);
 }
