@@ -1,12 +1,12 @@
 /* xgettext Smalltalk backend.
-   Copyright (C) 2002-2003 Free Software Foundation, Inc.
+   Copyright (C) 2002-2003, 2005-2007 Free Software Foundation, Inc.
 
    This file was written by Bruno Haible <haible@clisp.cons.org>, 2002.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,13 +14,15 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
 
+/* Specification.  */
+#include "x-smalltalk.h"
+
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -30,7 +32,6 @@
 #include "x-smalltalk.h"
 #include "error.h"
 #include "xalloc.h"
-#include "exit.h"
 #include "gettext.h"
 
 #define _(s) gettext(s)
@@ -148,7 +149,7 @@ comment_line_end ()
       buffer = xrealloc (buffer, bufmax);
     }
   buffer[buflen] = '\0';
-  xgettext_comment_add (buffer);
+  savable_comment_add (buffer);
 }
 
 
@@ -242,7 +243,7 @@ phase2_get (token_ty *tp)
 
 	case '\n':
 	  if (last_non_comment_line > last_comment_line)
-	    xgettext_comment_reset ();
+	    savable_comment_reset ();
 	  /* FALLTHROUGH */
 	case ' ':
 	case '\t':
@@ -323,7 +324,7 @@ phase2_get (token_ty *tp)
 	      case '@':
 	      case '?':
 	      case '%':
-		name = xmalloc (3);
+		name = XNMALLOC (3, char);
 		name[0] = c;
 		name[1] = c2;
 		name[2] = '\0';
@@ -334,7 +335,7 @@ phase2_get (token_ty *tp)
 		phase1_ungetc (c2);
 		break;
 	      }
-	    name = xmalloc (2);
+	    name = XNMALLOC (2, char);
 	    name[0] = c;
 	    name[1] = '\0';
 	    tp->type = token_type_symbol;
@@ -541,7 +542,8 @@ extract_smalltalk (FILE *f,
 		lex_pos_ty pos;
 		pos.file_name = logical_file_name;
 		pos.line_number = token.line_number;
-		remember_a_message (mlp, token.string, null_context, &pos);
+		remember_a_message (mlp, NULL, token.string, null_context,
+				    &pos, savable_comment);
 		state = 0;
 		break;
 	      }
@@ -550,8 +552,9 @@ extract_smalltalk (FILE *f,
 		lex_pos_ty pos;
 		pos.file_name = logical_file_name;
 		pos.line_number = token.line_number;
-		plural_mp = remember_a_message (mlp, token.string,
-						null_context, &pos);
+		plural_mp = remember_a_message (mlp, NULL, token.string,
+						null_context, &pos,
+						savable_comment);
 		state = 4;
 		break;
 	      }
@@ -561,7 +564,8 @@ extract_smalltalk (FILE *f,
 		pos.file_name = logical_file_name;
 		pos.line_number = token.line_number;
 		remember_a_message_plural (plural_mp, token.string,
-					   null_context, &pos);
+					   null_context, &pos,
+					   savable_comment);
 		state = 0;
 		break;
 	      }
