@@ -1,12 +1,12 @@
 /* xgettext Tcl backend.
-   Copyright (C) 2002-2003, 2005 Free Software Foundation, Inc.
+   Copyright (C) 2002-2003, 2005-2007 Free Software Foundation, Inc.
 
    This file was written by Bruno Haible <haible@clisp.cons.org>, 2002.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,13 +14,15 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
 
+/* Specification.  */
+#include "x-tcl.h"
+
 #include <assert.h>
 #include <errno.h>
 #include <limits.h>
@@ -34,11 +36,10 @@
 #include "x-tcl.h"
 #include "error.h"
 #include "xalloc.h"
-#include "exit.h"
 #include "hash.h"
 #include "c-ctype.h"
 #include "po-charset.h"
-#include "ucs4-utf8.h"
+#include "unistr.h"
 #include "gettext.h"
 
 #define _(s) gettext(s)
@@ -87,23 +88,19 @@ x_tcl_keyword (const char *name)
   else
     {
       const char *end;
-      int argnum1;
-      int argnum2;
+      struct callshape shape;
 
       if (keywords.table == NULL)
-	init_hash (&keywords, 100);
+	hash_init (&keywords, 100);
 
-      split_keywordspec (name, &end, &argnum1, &argnum2);
+      split_keywordspec (name, &end, &shape);
 
       /* The characters between name and end should form a valid Tcl
 	 function name.  A leading "::" is redundant.  */
       if (end - name >= 2 && name[0] == ':' && name[1] == ':')
 	name += 2;
 
-      if (argnum1 == 0)
-	argnum1 = 1;
-      insert_entry (&keywords, name, end - name,
-		    (void *) (long) (argnum1 + (argnum2 << 10)));
+      insert_keyword_callshape (&keywords, name, end - name, &shape);
     }
 }
 
@@ -114,6 +111,8 @@ init_keywords ()
 {
   if (default_keywords)
     {
+      /* When adding new keywords here, also update the documentation in
+	 xgettext.texi!  */
       x_tcl_keyword ("::msgcat::mc");
       default_keywords = false;
     }
@@ -334,7 +333,7 @@ static inline void
 init_token (struct token *tp)
 {
   tp->allocated = 10;
-  tp->chars = (char *) xmalloc (tp->allocated * sizeof (char));
+  tp->chars = XNMALLOC (tp->allocated, char);
   tp->charcount = 0;
 }
 
@@ -393,7 +392,7 @@ comment_line_end ()
       buffer = xrealloc (buffer, bufmax);
     }
   buffer[buflen] = '\0';
-  xgettext_comment_add (buffer);
+  savable_comment_add (buffer);
 }
 
 
@@ -452,7 +451,7 @@ string_of_word (const struct word *wp)
   if (!(wp->type == t_string))
     abort ();
   n = wp->token->charcount;
-  str = (char *) xmalloc (n + 1);
+  str = XNMALLOC (n + 1, char);
   memcpy (str, wp->token->chars, n);
   str[n] = '\0';
   return str;
@@ -740,7 +739,7 @@ read_word (struct word *wp, int looking_for, flag_context_ty context)
       /* Comments assumed to be grouped with a message must immediately
 	 precede it, with no non-whitespace token on a line between both.  */
       if (last_non_comment_line > last_comment_line)
-	xgettext_comment_reset ();
+	savable_comment_reset ();
       wp->type = t_separator;
       return;
     }
@@ -780,7 +779,7 @@ read_word (struct word *wp, int looking_for, flag_context_ty context)
     }
 
   wp->type = t_string;
-  wp->token = (struct token *) xmalloc (sizeof (struct token));
+  wp->token = XMALLOC (struct token);
   init_token (wp->token);
   wp->line_number_at_start = line_number;
 
@@ -853,9 +852,8 @@ read_command (int looking_for, flag_context_ty outer_context)
   {
     int arg = 0;		/* Current argument number.  */
     flag_context_list_iterator_ty context_iter;
-    int argnum1 = 0;		/* First string position.  */
-    int argnum2 = 0;		/* Plural string position.  */
-    message_ty *plural_mp = NULL;	/* Remember the msgid.  */
+    const struct callshapes *shapes = NULL;
+    struct arglist_parser *argparser = NULL;
 
     for (;; arg++)
       {
@@ -875,7 +873,11 @@ read_command (int looking_for, flag_context_ty outer_context)
 	/* Recognize end of command.  */
 	if (inner.type == t_separator || inner.type == t_bracket
 	    || inner.type == t_brace || inner.type == t_eof)
-	  return inner.type;
+	  {
+	    if (argparser != NULL)
+	      arglist_parser_done (argparser, arg);
+	    return inner.type;
+	  }
 
 	if (extract_all)
 	  {
@@ -885,8 +887,8 @@ read_command (int looking_for, flag_context_ty outer_context)
 
 		pos.file_name = logical_file_name;
 		pos.line_number = inner.line_number_at_start;
-		remember_a_message (mlp, string_of_word (&inner),
-				    inner_context, &pos);
+		remember_a_message (mlp, NULL, string_of_word (&inner),
+				    inner_context, &pos, savable_comment);
 	      }
 	  }
 
@@ -904,14 +906,13 @@ read_command (int looking_for, flag_context_ty outer_context)
 		if (function_name[0] == ':' && function_name[1] == ':')
 		  stripped_name += 2;
 
-		if (find_entry (&keywords,
-				stripped_name, strlen (stripped_name),
-				&keyword_value)
+		if (hash_find_entry (&keywords,
+				     stripped_name, strlen (stripped_name),
+				     &keyword_value)
 		    == 0)
-		  {
-		    argnum1 = (int) (long) keyword_value & ((1 << 10) - 1);
-		    argnum2 = (int) (long) keyword_value >> 10;
-		  }
+		  shapes = (const struct callshapes *) keyword_value;
+
+		argparser = arglist_parser_alloc (mlp, shapes);
 
 		context_iter =
 		  flag_context_list_iterator (
@@ -926,36 +927,14 @@ read_command (int looking_for, flag_context_ty outer_context)
 	  }
 	else
 	  {
-	    /* These are the argument positions.
-	       Extract a string if we have reached the right
-	       argument position.  */
-	    if (arg == argnum1)
-	      {
-		if (inner.type == t_string)
-		  {
-		    lex_pos_ty pos;
-		    message_ty *mp;
-
-		    pos.file_name = logical_file_name;
-		    pos.line_number = inner.line_number_at_start;
-		    mp = remember_a_message (mlp, string_of_word (&inner),
-					     inner_context, &pos);
-		    if (argnum2 > 0)
-		      plural_mp = mp;
-		  }
-	      }
-	    else if (arg == argnum2)
-	      {
-		if (inner.type == t_string && plural_mp != NULL)
-		  {
-		    lex_pos_ty pos;
-
-		    pos.file_name = logical_file_name;
-		    pos.line_number = inner.line_number_at_start;
-		    remember_a_message_plural (plural_mp, string_of_word (&inner),
-					       inner_context, &pos);
-		  }
-	      }
+	    /* These are the argument positions.  */
+	    if (argparser != NULL && inner.type == t_string)
+	      arglist_parser_remember (argparser, arg,
+				       string_of_word (&inner),
+				       inner_context,
+				       logical_file_name,
+				       inner.line_number_at_start,
+				       savable_comment);
 	  }
 
 	free_word (&inner);
